@@ -2,10 +2,10 @@
 #'
 #' We consider the k sample multinomial problem where we observe k vectors
 #' (possibly of different lengths), each representing an independent sample from
-#' a multinomial. For a given function psi which takes in the concatenated
-#' vector of multinomial probabilities and outputs a real number, we are
-#' interested in computing a p-value for a test of psi >= psi0, and constructing
-#' a confidence interval for psi.
+#' a multinomial. For a given function \eqn{\tau(\theta)} which takes in the concatenated
+#' vector of multinomial probabilities \eqn{\theta} and outputs a real number, we are
+#' interested in computing a p-value for a test of \eqn{\tau(\theta) = \psi \geq \psi_0}, and constructing
+#' a confidence interval for \eqn{\psi}.
 #'
 #' Let \eqn{T_j} be distributed
 #' \eqn{\mbox{Multinomial}_{d_j}(\boldsymbol{\theta}_j, n_j)} for \eqn{j = 1,
@@ -17,12 +17,12 @@
 #' \eqn{\boldsymbol{X} = (X_1, \ldots, X_k)}, which is a vector of counts obtained
 #' by concatenating the k independent count vectors, let \eqn{G(\boldsymbol{X})}
 #' denote a real-valued statistic that defines the ordering of the sample space.
-#' Tne default choice of the statistic is to estimate \eqn{\boldsymbol{\theta}}
+#' The default choice of the statistic is to estimate \eqn{\boldsymbol{\theta}}
 #' with the sample proportions and plug them into \eqn{\tau(\boldsymbol{\theta})}.
 #' This function calculates a p value for a test of the null hypothesis
-#' \eqn{H_0: \psi(\boldsymbol{\theta}) \neq \psi_0} for the two sided case,
-#' \eqn{H_0: \psi(\boldsymbol{\theta}) \leq \psi_0} for the case \code{alternative = "greater"}, and
-#' \eqn{H_0: \psi(\boldsymbol{\theta}) \geq \psi_0} for the case \code{alternative = "less"}.
+#' \eqn{H_0: \psi \neq \psi_0} for the two sided case,
+#' \eqn{H_0: \psi \leq \psi_0} for the case \code{alternative = "greater"}, and
+#' \eqn{H_0: \psi \geq \psi_0} for the case \code{alternative = "less"}.
 #' We make no assumptions and do not rely on large sample approximations.
 #' It also optionally constructs a \eqn{1 - \alpha} percent confidence interval for \eqn{\psi}.
 #' The computation is somewhat involved so it is best for small sample sizes. The
@@ -32,16 +32,16 @@
 #' is basically the definition of a p-value implemented with Monte Carlo methods. Some
 #' options for speeding up the calculation are available.
 #'
-#' @section Specifying the function psi:
-#' The psi parameter should be a function that either: 1) takes a vector of length
-#' sum(d_j) (the total number of bins) and outputs a single number, or 2) takes a
+#' @section Specifying the function \eqn{\tau(\cdot)}:
+#' This function is the \code{f_param} argument and should be a function that either: 1)
+#' takes a vector of length sum(d_j) (the total number of bins) and outputs a single number, or 2) takes a
 #' matrix with number of columns equal to sum(d_j), and arbitrary number of rows and
 #' outputs a vector with length equal to the number of rows. In other words, psi can be
 #' not vectorized or it can be vectorized by row. Writing it so that it is vectorized
 #' can speed up the calculation. See examples.
 #'
 #' @section Boundary issues:
-#' It is required to provide psi_limits, a vector of length 2 giving the
+#' It is required to provide \code{psi_limits}, a vector of length 2 giving the
 #' smallest and largest possible values that the function psi can take, e.g., \code{c(0, 1)}.
 #' If the null hypothesis value psi0 is at one of the limits, it is often the case
 #' that sampling from the null parameter space is impossible because it is a set of
@@ -73,27 +73,28 @@
 #'
 #' @param data A list with k elements representing the vectors of counts of a
 #'   k-sample multinomial
-#' @param psi Function that takes in parameters and outputs a real
+#' @param f_param Function that takes in parameters and outputs psi, a real
 #'   valued number for each parameter. Can be vectorized rowwise for a matrix or not.
-#' @param statistic Function that takes in a matrix with data vectors in the rows, and outputs a vector with the number of rows in the matrix. If NULL, will be inferred from psi by plugging in the empirical proportions.
+#' @param statistic Function that takes in a matrix with data vectors in the rows, and outputs a vector with the number of rows in the matrix. If NULL, will be inferred from \code{f_param} by plugging in the empirical proportions.
 #' @param psi0 The null hypothesis value for the parameter being tested.
 #' @param alternative a character string specifying the alternative hypothesis, must be one of "two.sided" (default), "greater" or "less"
 #' @param psi_limits A vector of length 2 giving the lower and upper limits of
-#'   the range of \eqn{\psi(\theta)}
-#' @param theta_null_points An optional matrix where each row is a theta value that gives psi(theta) = psi0. If this is supplied and psi0 = one of the boundary points, then a truly exact p-value will be calculated.
+#'   the range of \eqn{\tau(\theta)}
+#' @param theta_null_points An optional matrix where each row is a theta value that gives \code{f_param(theta) = psi0}. If this is supplied and psi0 = one of \code{psi_limits}, then a truly exact p-value will be calculated.
 #' @param p_target If a p-value is found that is greater than p_target, terminate the algorithm early.
 #' @param conf_int If TRUE, calculates a confidence interval by inverting the p-value function
 #' @param conf_level A number between 0 and 1, the confidence level.
 #' @param itp_maxit Maximum iterations to use in the ITP algorithm. Only relevant if conf_int = TRUE.
 #' @param itp_eps Epsilon value to use for the ITP algorithm. Only relevant if conf_int = TRUE.
-#' @param p_value_limits A vector of length 2 giving lower bounds on the p-values corresponding to psi0 at psi_limits. Only relvant if conf_int = TRUE.
+#' @param p_value_limits A vector of length 2 giving lower bounds on the one-sided p-values corresponding to psi0 = psi_limits, with alternative = "less" and "greater", respectively. Only relevant if conf_int = TRUE. See examples.
 #' @param maxit Maximum number of iterations of the Monte Carlo procedure
 #' @param chunksize The number of samples to take from the parameter space at each iteration
-#' @param theta_sampler Function to take samples from the \eqn{Theta} parameter space. Default is \link{runif_dk_vects}.
+#' @param theta_sampler Function to take samples from the \eqn{Theta} parameter space. Default is \link{runif_dk_vects}. Must be a function of two parameters \code{d_k} a vector of dimensions, and \code{chunksize} the number of samples to take, and return a matrix with \code{sum(d_k)} columns and \code{chunksize} rows. See examples.
 #' @param ga Logical, if TRUE, uses gradient ascent.
 #' @param ga_gfactor Concentration parameter scale in the gradient ascent algorithm. A number or "adapt"
 #' @param ga_lrate The gradient ascent learning rate
 #' @param ga_restart_every Restart the gradient ascent after this number of iterations at a sample from \code{theta_sampler}
+#' @param seed Seed for the random number generator. Can be set to NULL in which case no seed is set.
 #'
 #' @returns An object of class "htest", which is a list with the following elements:
 #' \describe{
@@ -112,45 +113,54 @@
 #'
 #' @export
 #' @examples
-#' psi_ba <- function(theta) {
+#' tau_ba <- function(theta) {
 #'   theta1 <- theta[1:4]
 #'   theta2 <- theta[5:8]
 #'   sum(sqrt(theta1 * theta2))
 #'   }
 #' data <- list(T1 = c(2,1,2,1), T2 = c(0,1,3,3))
-#' xactonomial(data, psi_ba, psi_limits = c(0, 1), psi0 = .5,
+#' xactonomial(data, tau_ba, psi_limits = c(0, 1), psi0 = .5,
 #'   conf_int = FALSE, maxit = 15, chunksize = 200)
 #'
 #' # vectorized by row
-#' psi_ba_v <- function(theta) {
+#' tau_ba_v <- function(theta) {
 #' theta1 <- theta[,1:4, drop = FALSE]
 #' theta2 <- theta[,5:8, drop = FALSE]
 #' rowSums(sqrt(theta1 * theta2))
 #' }
 #' data <- list(T1 = c(2,1,2,1), T2 = c(0,1,3,3))
-#' xactonomial(data, psi_ba_v, psi_limits = c(0, 1), psi0 = .5,
+#' xactonomial(data, tau_ba_v, psi_limits = c(0, 1), psi0 = .5,
 #'  conf_int = FALSE, maxit = 10, chunksize = 200)
 #'
 #'  # example of using theta_null_points
 #'  # psi = 1/3 occurs when all probs = 1/3
-#'  psi_max <- function(pp) {
+#'  tau_max <- function(pp) {
 #'    max(pp)
 #'  }
 #'
 #' data <- list(c(13, 24, 13))
 #'
-#' xactonomial(data, psi_max, psi_limits = c(1 / 3, 1), psi0 = 1/ 3,
+#' xactonomial(data, tau_max, psi_limits = c(1 / 3, 1), psi0 = 1/ 3,
 #'   conf_int = FALSE, theta_null_points = t(c(1/3, 1/3, 1/3)))
 #'
 #' ## in this case using p_value_limits improves confidence interval performance
 #'
-#'  xactonomial(data, psi_max, psi_limits = c(1 / 3, 1), psi0 = 1/ 3,
+#'  xactonomial(data, tau_max, psi_limits = c(1 / 3, 1), psi0 = 1/ 3,
 #'   conf_int = TRUE, theta_null_points = t(c(1/3, 1/3, 1/3)),
 #'   p_value_limits = c(.1, 1e-8))
 #'
+#' ## specifying theta_sampler
+#'
+#' dirich_sampler <- function(d_k, chunksize){
+#'  rdirich_dk_vects(chunksize, list(1:4 + 1))
+#' }
+#'
+#' xactonomial(list(1:4),tau_max,
+#'             psi_limits = c(0.25,1), psi0 = .5, conf_int = FALSE,
+#'             theta_sampler = dirich_sampler)
 #'
 
-xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
+xactonomial <- function(data, f_param, statistic = NULL, psi0 = NULL,
                         alternative = c("two.sided", "less", "greater"),
                         psi_limits, theta_null_points = NULL, p_target = 1,
                         conf_int = TRUE, conf_level = .95, itp_maxit = 10, itp_eps = .005,
@@ -158,12 +168,13 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
                         maxit = 50, chunksize = 500,
                         theta_sampler = runif_dk_vects,
                         ga = TRUE, ga_gfactor = "adapt", ga_lrate = .01,
-                        ga_restart_every = 10
+                        ga_restart_every = 10,
+                        seed = 503
                         ) {
 
   alternative <- match.arg(alternative)
   if(missing(psi_limits)) {
-    stop("Please provide the lower and upper limits of the output of the psi function as a vector in psi_limits")
+    stop("Please provide the lower and upper limits of the output of the f_param function as a vector in psi_limits")
   }
 
   if(!is.null(psi0)) {
@@ -171,10 +182,16 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
     stop("psi0 outside the possible range specified by psi_limits.")
   }}
 
-  if(!is.null(psi0) & (any(abs(psi0 - psi_limits) < 1e-8) & is.null(theta_null_points))) {
+  if(!is.null(psi0) & (any(abs(psi0 - psi_limits) < 1e-8) &
+                       (is.null(theta_null_points) & isTRUE(all.equal(theta_sampler, runif_dk_vects))))) {
 
 
-    warning("The psi0 value is on the boundary of the psi parameter space. The Monte Carlo method may not perform well in this case. Consider specifying theta_null_points which is a matrix of theta vectors at which psi(theta) = psi0.")
+    warning("The psi0 value is on the boundary of the psi parameter space. The default Monte Carlo method may not perform well in this case. Consider specifying theta_null_points which is a matrix of theta vectors at which f_param(theta) = psi0, or specifying theta_sampler to sample directly from the null space.")
+
+  }
+
+  if(!is.null(seed)) {
+    set.seed(seed)
   }
 
   alpha <- 1 - conf_level
@@ -184,7 +201,7 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
   n_k <- sapply(data, sum)
 
   vtest <- runif_dk_vects(d_k, 5)
-  psi_is_vectorized <- tryCatch(length(psi(vtest)) == 5, error = function(e) FALSE)
+  f_is_vectorized <- tryCatch(length(f_param(vtest)) == 5, error = function(e) FALSE)
 
   sspace_size <- prod(sapply(data, \(dd) choose(length(dd) + sum(dd) - 1, length(dd) - 1)))
 
@@ -214,35 +231,38 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
   }
 
 
-  if(is.null(statistic)) { ## compute psi with empirical proportions
+  if(is.null(statistic)) { ## compute f_param with empirical proportions
 
+    user_stat <- FALSE
     statistic <- function(df) {
       denom <- rep.int(n_k, d_k)
       if(is.matrix(df)) {
         pmat <- matrix(rep(denom, nrow(df)), nrow = nrow(df), ncol = length(denom), byrow = TRUE)
-        if(psi_is_vectorized) {
-          psi(df / pmat)
+        if(f_is_vectorized) {
+          f_param(df / pmat)
         } else {
-          apply(df / pmat, 1, psi)
+          apply(df / pmat, 1, f_param)
         }
       } else {
-        if(psi_is_vectorized) {
-          psi(t(df / denom))
+        if(f_is_vectorized) {
+          f_param(t(df / denom))
         } else {
-          psi(df / denom)
+          f_param(df / denom)
         }
 
       }
 
     }
 
+  } else {
+    user_stat <- TRUE
   }
 
   psi_obs <- statistic(unlist(data))
 
   psi_hat <- statistic(spacelist$Sspace)
 
-  method <- "Monte-Carlo multinomial test"
+  method <- "Monte Carlo multinomial test"
   pvalues <- if(!is.null(psi0)) {
 
     if(!is.null(theta_null_points) && any(abs(psi0 - psi_limits) < 1e-8)) {
@@ -257,10 +277,11 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
     } else {
 
 
-    pvalue_psi0(psi0 = psi0, psi = psi, psi_hat = psi_hat, psi_obs = psi_obs, alternative = alternative,
+    pvalue_psi0(psi0 = psi0, f_param = f_param, psi_hat = psi_hat, psi_obs = psi_obs,
+                alternative = alternative,
                 maxit = maxit, chunksize = chunksize,
                             p_target = p_target, SSpacearr = spacelist$Sspace, logC = spacelist$logC,
-                            d_k = d_k, psi_is_vectorized = psi_is_vectorized,
+                            d_k = d_k, f_is_vectorized = f_is_vectorized,
                             theta_sampler = theta_sampler,
                             ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
                 ga_restart_every = ga_restart_every)
@@ -271,14 +292,17 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
 
 
   confint <- if(conf_int) {
-  flower <- function(x, psi, psi_hat, psi_obs, maxit, chunksize, p_target, SSpacearr, logC, d_k, psi_is_vectorized,
+  flower <- function(x, f_param, psi_hat, psi_obs, maxit, chunksize, p_target, SSpacearr,
+                     logC, d_k, f_is_vectorized,
                      theta_sampler, ga, ga_gfactor, ga_lrate, ga_restart_every){
-    pvalue_psi0(x, psi, psi_hat, psi_obs, maxit = maxit, chunksize = chunksize,
+    pest <- pvalue_psi0(x, f_param, psi_hat, psi_obs, maxit = maxit, chunksize = chunksize,
                 p_target = p_target, SSpacearr = SSpacearr, logC = logC,
-                d_k = d_k, psi_is_vectorized = psi_is_vectorized,
-                theta_sampler = theta_sampler,
+                d_k = d_k, f_is_vectorized = f_is_vectorized,
+                theta_sampler = theta_sampler, alternative = "greater",
                 ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
-                ga_restart_every = ga_restart_every)[1] - alpha / 2}
+                ga_restart_every = ga_restart_every, warn = FALSE)[1]
+    (if(is.na(pest)) 0 else pest) - alpha / 2
+    }
 
   if(!is.null(p_value_limits)){
     flower.boundary <- p_value_limits[1] - alpha / 2
@@ -297,14 +321,15 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
   }
 
 
-  fupper <- function(x, psi, psi_hat, psi_obs, maxit, chunksize, p_target, SSpacearr, logC, d_k, psi_is_vectorized,
+  fupper <- function(x, f_param, psi_hat, psi_obs, maxit, chunksize, p_target, SSpacearr, logC, d_k, f_is_vectorized,
                      theta_sampler, ga, ga_gfactor, ga_lrate, ga_restart_every){
-    pvalue_psi0(x, psi, psi_hat, psi_obs, maxit = maxit, chunksize = chunksize,
+    pest <- pvalue_psi0(x, f_param, psi_hat, psi_obs, maxit = maxit, chunksize = chunksize,
                 p_target = p_target, SSpacearr = SSpacearr, logC = logC,
-                d_k = d_k, psi_is_vectorized = psi_is_vectorized,
-                theta_sampler = theta_sampler,
+                d_k = d_k, f_is_vectorized = f_is_vectorized,
+                theta_sampler = theta_sampler, alternative = "less",
                 ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
-                ga_restart_every = ga_restart_every)[2] - alpha / 2
+                ga_restart_every = ga_restart_every, warn = FALSE)[2]
+    (if(is.na(pest)) 0 else pest) - alpha / 2
   }
   if(!is.null(p_value_limits)){
     fupper.boundary <- p_value_limits[2] - alpha / 2
@@ -328,11 +353,11 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
     lower_limit <- itp_root(flower, psi_limits[1], psi_limits[2],
                             fa = flower.boundary, fb = 1 - alpha / 2, maxiter = itp_maxit,
                             eps = itp_eps,
-                            psi = psi, psi_hat = psi_hat, psi_obs = psi_obs,
+                            f_param = f_param, psi_hat = psi_hat, psi_obs = psi_obs,
                             maxit = maxit, chunksize = chunksize,
                             p_target = alpha / 2 + 2 * itp_eps,
                             SSpacearr = spacelist$Sspace, logC = spacelist$logC,
-                            d_k = d_k, psi_is_vectorized = psi_is_vectorized,
+                            d_k = d_k, f_is_vectorized = f_is_vectorized,
                             theta_sampler = theta_sampler,
                             ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
                             ga_restart_every = ga_restart_every)
@@ -351,11 +376,11 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
     upper_limit <- itp_root(fupper, psi_limits[1], psi_limits[2],
                             fa = 1-alpha / 2, fb = fupper.boundary, maxiter = itp_maxit,
                             eps = itp_eps,
-                            psi = psi, psi_hat = psi_hat, psi_obs = psi_obs,
+                            f_param = f_param, psi_hat = psi_hat, psi_obs = psi_obs,
                             maxit = maxit, chunksize = chunksize,
                             p_target = alpha / 2 + 2 * itp_eps,
                             SSpacearr = spacelist$Sspace, logC = spacelist$logC,
-                            d_k = d_k, psi_is_vectorized = psi_is_vectorized,
+                            d_k = d_k, f_is_vectorized = f_is_vectorized,
                             theta_sampler = theta_sampler,
                             ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
                             ga_restart_every = ga_restart_every)
@@ -368,12 +393,51 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
   }
 
 
+  if(missing(p_value_limits) & is.null(theta_null_points)) {
+  lowerp <-   pvalue_psi0(psi0 = lower_limit, f_param = f_param, psi_hat = psi_hat, psi_obs = psi_obs,
+                         # alternative = "less",
+                          maxit = maxit, chunksize = chunksize,
+                          p_target = p_target, SSpacearr = spacelist$Sspace, logC = spacelist$logC,
+                          d_k = d_k, f_is_vectorized = f_is_vectorized,
+                          theta_sampler = theta_sampler,
+                          ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
+                          ga_restart_every = ga_restart_every, warn = FALSE)[1]
+
+
+  if(lowerp > alpha/ 2) {
+    warning("No root found for lower confidence limit, using lower boundary.")
+    lower_limit <- psi_limits[1]
+
+  }
+
+  upperp <-   pvalue_psi0(psi0 = upper_limit, f_param = f_param, psi_hat = psi_hat, psi_obs = psi_obs,
+                          #alternative = "greater",
+                          maxit = maxit, chunksize = chunksize,
+                          p_target = p_target, SSpacearr = spacelist$Sspace, logC = spacelist$logC,
+                          d_k = d_k, f_is_vectorized = f_is_vectorized,
+                          theta_sampler = theta_sampler,
+                          ga = ga, ga_gfactor = ga_gfactor, ga_lrate = ga_lrate,
+                          ga_restart_every = ga_restart_every, warn = FALSE)[2]
+
+
+  if(upperp > alpha/ 2) {
+    warning("No root found for upper confidence limit, using upper boundary.")
+    upper_limit <- psi_limits[2]
+  }
+  }
+
   c(lower_limit, upper_limit)
   } else c(NA, NA)
 
   p.sequence <- attr(pvalues, "p.sequence")
 
   attr(confint, "conf.level") = 1 - alpha
+  f_pname <- if(!user_stat) {
+    deparse1(substitute(f_param))
+  } else {
+    deparse1(substitute(statistic))
+  }
+  names(psi_obs) <- f_pname
 
   res <- list(
     estimate = psi_obs,
@@ -392,10 +456,10 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
 }
 
 
-#' Compute a p value for the test of psi <= psi0 (lower = TRUE) or psi >= psi0 (lower = FALSE)
+#' Compute a p value for the test of psi <= psi0 and/or psi >= psi0
 #'
 #' @param psi0 The null hypothesis value for the parameter being tested.
-#' @param psi Function that takes in parameters and outputs a real
+#' @param f_param Function that takes in parameters and outputs a real
 #'   valued number for each parameter. Can be vectorized rowwise for a matrix or not.
 #' @param psi_hat The vector of psi values at each element of the sample space
 #' @param psi_obs The observed estimate at the given data
@@ -406,7 +470,7 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
 #' @param SSpacearr The sample space matrix
 #' @param logC The log multinomial coefficients for each row of the sample space
 #' @param d_k The vector of dimensions
-#' @param psi_is_vectorized Is psi vectorized by row?
+#' @param f_is_vectorized Is f_param vectorized by row?
 #' @param theta_sampler Function to take samples from the \eqn{Theta} parameter space. Default is \link{runif_dk_vects}.
 #' @param ga Logical, if TRUE, uses gradient ascent.
 #' @param ga_gfactor Concentration parameter scale in the gradient ascent algorithm. A number or "adapt"
@@ -418,32 +482,32 @@ xactonomial <- function(data, psi, statistic = NULL, psi0 = NULL,
 #' @examples
 #'
 #' sspace_3_5 <- matrix(sspace_multinom(3, 5), ncol = 3, byrow = TRUE)
-#' psi <- function(theta) max(theta)
+#' f_max <- function(theta) max(theta)
 #' logC <- apply(sspace_3_5, 1, log_multinom_coef, sumx = 5)
-#' psi_hat <- apply(sspace_3_5, 1, \(x) psi(x / sum(x)))
-#' pvalue_psi0(.3, psi, psi_hat, .4, maxit = 10, chunksize = 100,
+#' psi_hat <- apply(sspace_3_5, 1, \(x) f_max(x / sum(x)))
+#' pvalue_psi0(.3, f_max, psi_hat, .4, maxit = 10, chunksize = 100,
 #'  p_target = 1, SSpacearr = sspace_3_5, logC = logC, d_k = 3, warn = FALSE)
 
-pvalue_psi0 <- function(psi0, psi, psi_hat, psi_obs, alternative = "two.sided",
+pvalue_psi0 <- function(psi0, f_param, psi_hat, psi_obs, alternative = "two.sided",
                         maxit, chunksize,
                         p_target,
-                        SSpacearr, logC, d_k, psi_is_vectorized = FALSE,
+                        SSpacearr, logC, d_k, f_is_vectorized = FALSE,
                         theta_sampler = runif_dk_vects,
                         ga = FALSE, ga_gfactor = 1, ga_lrate = .01,
                         ga_restart_every = 10, warn = TRUE
                         ) {
 
-  already_warned <- !FALSE
+  already_warned <- !warn
   II.lower <- psi_hat >= psi_obs
   II.upper <- psi_hat <= psi_obs
 
-  p.null <- rep(1 / (maxit * chunksize), maxit)
-  p.alt <- rep(1 / (maxit * chunksize), maxit)
+  p.null <- rep(0, maxit)
+  p.alt <- rep(0, maxit)
 
   theta_cands <- theta_sampler(d_k, chunksize)
   null_continue <- alt_continue <- TRUE
   if(alternative == "greater") alt_continue <- FALSE
-  if(alternative == "lower") null_continue <- FALSE
+  if(alternative == "less") null_continue <- FALSE
   null_stop <- alt_stop <- maxit
   never_null <- null_continue
   never_alt <- alt_continue
@@ -452,7 +516,7 @@ pvalue_psi0 <- function(psi0, psi, psi_hat, psi_obs, alternative = "two.sided",
     if(isFALSE(null_continue) & isFALSE(alt_continue)) break
 
     this_theta <- theta_cands
-    psi_theta <- if(psi_is_vectorized) psi(theta_cands) else apply(theta_cands, MARGIN = 1, psi)
+    psi_theta <- if(f_is_vectorized) f_param(theta_cands) else apply(theta_cands, MARGIN = 1, f_param)
     null_indicator <- psi_theta <= psi0
 
     if(i %% ga_restart_every == 0) {
@@ -544,10 +608,13 @@ pvalue_psi0 <- function(psi0, psi, psi_hat, psi_obs, alternative = "two.sided",
   }
   if((isTRUE(never_null) | isTRUE(never_alt)) & !already_warned) {
     already_warned <- TRUE
-    warning("Never found a value from the null space, results are probably not valid! Increase iterations or specify theta_null_points")
+    warning("Never found a value from the null space, results are not valid! Increase iterations, specify theta_null_points, or specify theta_sampler")
   }
 
-  res <- c(null = max(p.null, na.rm = TRUE), alt = max(p.alt, na.rm = TRUE))
+  outnull <- if(isTRUE(never_null)) NA else max(p.null, na.rm = TRUE)
+  outalt <- if(isTRUE(never_alt)) NA else max(p.alt, na.rm = TRUE)
+
+  res <- c(null = outnull, alt = outalt)
   attr(res, "p.sequence") <- list(p.null = p.null[1:null_stop], p.alt = p.alt[1:alt_stop])
   res
 }
